# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Nova logging handler.

This module adds to logging functionality by adding the option to specify
a context object when calling the various log methods.  If the context object
is not specified, default formatting is used.
"""


import functools
import logging
from nova import flags

flags.DEFINE_string('logging_context_format_string',
                    '(%(name)s): %(levelname)s [%(request_id)s %(user_id)s '
                    '%(project_id)s] %(message)s',
                    'format string to use for log messages')

flags.DEFINE_string('logging_default_format_string',
                    '(%(name)s): %(levelname)s [N/A] %(message)s',
                    'format string to use for log messages')

flags.DEFINE_list('default_log_levels',
                  ['amqplib=WARN','audit=INFO'],
                  'list of logger=LEVEL pairs')

def _set_log_levels():
    """This conveniencemethod is called at the end of the import"""
    for pair in FLAGS.default_log_levels:
        logger, _sep, level = pair.partition('=')
        level = logging._levelNames.get(level, logging.NOTSET)
        logging.getLogger(logger).setLevel(level)

# NOTE(vish): This replicates all of the currently used objects in the logging
#            namespace, but more will need to be added if additional
#            functionality is desired.
FLAGS = flags.FLAGS
CRITICAL = logging.CRITICAL
FATAL = logging.FATAL
ERROR = logging.ERROR
WARNING = logging.WARNING
WARN = logging.WARN
INFO = logging.INFO
DEBUG = logging.DEBUG
NOTSET = logging.NOTSET
StreamHandler = logging.StreamHandler


class NovaFormatter(logging.Formatter):
    """Formatter that supports two format strings"""

    def format(self, record):
        """Uses contextstring if request_id is set, otherwise default"""
        if record.__dict__.get('request_id', None):
            self._fmt = FLAGS.logging_context_format_string
        else:
            self._fmt = FLAGS.logging_default_format_string
        return logging.Formatter.format(self, record)

_formatter = NovaFormatter()


class LoggerWrapper(object):
    """Wrapper that allows for context kwarg to be passed in"""

    def __init__(self, logger):
        self.logger = logger

    def __getattr__(self, key):
        """Delegates logging comands to a logger adapter with context"""
        if key in ['debug',
                   'info',
                   'warning',
                   'error',
                   'exception',
                   'log']:
            return functools.partial(self.delegate, key)
        else:
            return getattr(self.logger, key)

    def delegate(self, key, *args, **kwargs):
        """Call the proper method on the Adapter"""
        context = _extract_context(kwargs)
        adapter = logging.LoggerAdapter(self.logger, context)
        getattr(adapter, key)(*args, **kwargs)


def getLogger(name=None):
    """Replicates logging getLogger with a wrapper to handle context"""
    logger = logging.getLogger(name)
    for handler in logger.handlers:
        handler.setFormatter(_formatter)
    return LoggerWrapper(logger)


def _ensure_root():
    """Ensures that root exists, it has a handler and has a formatter"""
    if len(logging.root.handlers) == 0:
        logging.basicConfig()
    if not _ensure_root.initted:
        for handler in logging.root.handlers:
            handler.setFormatter(_formatter)
        _ensure_root.initted = True


_ensure_root.initted = False


def audit(msg, *args, **kwargs):
    """Shortcut for logging to audit log with sevrity 'INFO'."""
    context = _extract_context(kwargs)
    logger = getLogger('audit')
    logging.LoggerAdapter(logger,
                          context).info(*((msg,)+args), **kwargs)


def _extract_context(kwargs):
    default_context = {'request_id': None,
                       'user_id': None,
                       'project_id': None}
    context = kwargs.pop('context', default_context)
    if not isinstance(context, dict) and getattr(context, 'to_dict', None):
        context = context.to_dict()
    return context


def critical(msg, *args, **kwargs):
    """
    Log a message with severity 'CRITICAL' on the root logger.
    """
    context = _extract_context(kwargs)
    _ensure_root()
    logging.LoggerAdapter(logging.root,
                          context).critical(*((msg,)+args), **kwargs)


fatal = critical


def error(msg, *args, **kwargs):
    """
    Log a message with severity 'ERROR' on the root logger.
    """
    context = _extract_context(kwargs)
    _ensure_root()
    logging.LoggerAdapter(logging.root,
                          context).error(*((msg,)+args), **kwargs)


def exception(msg, *args, **kwargs):
    """
    Log a message with severity 'ERROR' on the root logger,
    with exception information.
    """
    if kwargs is None:
        kwargs = {}
    kwargs.update({'exc_info': 1})
    error(msg, *args, **kwargs)


def warning(msg, *args, **kwargs):
    """
    Log a message with severity 'WARNING' on the root logger.
    """
    context = _extract_context(kwargs)
    _ensure_root()
    logging.LoggerAdapter(logging.root,
                          context).warning(*((msg,)+args), **kwargs)


warn = warning


def info(msg, *args, **kwargs):
    """
    Log a message with severity 'INFO' on the root logger.
    """
    context = _extract_context(kwargs)
    _ensure_root()
    logging.LoggerAdapter(logging.root,
                          context).info(*((msg,)+args), **kwargs)


def debug(msg, *args, **kwargs):
    """
    Log a message with severity 'DEBUG' on the root logger.
    """
    context = _extract_context(kwargs)
    _ensure_root()
    logging.LoggerAdapter(logging.root,
                          context).debug(*((msg,)+args), **kwargs)


def log(level, msg, *args, **kwargs):
    """
    Log 'msg % args' with the integer severity 'level' on the root logger.
    """
    context = _extract_context(kwargs)
    _ensure_root()
    logging.LoggerAdapter(logging.root,
                          context).log(*((level, msg)+args), **kwargs)

